/****************************************************************************
** Meta object code from reading C++ file 'database.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../alligator-26.07.90/src/database.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'database.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8DatabaseE_t {};
} // unnamed namespace

template <> constexpr inline auto Database::qt_create_metaobjectdata<qt_meta_tag_ZN8DatabaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Database",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "feedAdded",
        "",
        "url",
        "feedDetailsUpdated",
        "displayName",
        "description",
        "feedGroupsUpdated",
        "feedGroupRemoved",
        "groupName",
        "entryReadChanged",
        "entryId",
        "read",
        "feedReadChanged",
        "entryFavoriteChanged",
        "favorite",
        "addFeed",
        "markEntriesRead",
        "importFeeds",
        "path",
        "exportFeeds",
        "addFeedGroup",
        "name",
        "isDefault",
        "removeFeedGroup",
        "setDefaultGroup",
        "editFeed",
        "setRead",
        "setFeedRead",
        "setAllRead",
        "setFavorite"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'feedAdded'
        QtMocHelpers::SignalData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'feedDetailsUpdated'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'feedGroupsUpdated'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'feedGroupRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'entryReadChanged'
        QtMocHelpers::SignalData<void(const QString &, bool)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Signal 'feedReadChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'entryFavoriteChanged'
        QtMocHelpers::SignalData<void(const QString &, bool)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 19 },
        }}),
        // Method 'addFeed'
        QtMocHelpers::MethodData<void(const QString &, const QString &, const bool)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 13 }, { QMetaType::Bool, 21 },
        }}),
        // Method 'addFeed'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(20, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 13 },
        }}),
        // Method 'addFeed'
        QtMocHelpers::MethodData<void(const QString &)>(20, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'importFeeds'
        QtMocHelpers::MethodData<void(const QString &)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Method 'exportFeeds'
        QtMocHelpers::MethodData<void(const QString &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Method 'addFeedGroup'
        QtMocHelpers::MethodData<void(const QString &, const QString &, const int)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 10 }, { QMetaType::Int, 27 },
        }}),
        // Method 'addFeedGroup'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(25, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 10 },
        }}),
        // Method 'removeFeedGroup'
        QtMocHelpers::MethodData<void(const QString &)>(28, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Method 'setDefaultGroup'
        QtMocHelpers::MethodData<void(const QString &)>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Method 'editFeed'
        QtMocHelpers::MethodData<void(const QString &, const QString &, const QString &)>(30, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 9 }, { QMetaType::QString, 13 },
        }}),
        // Method 'setRead'
        QtMocHelpers::MethodData<void(const QString &, bool)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Method 'setFeedRead'
        QtMocHelpers::MethodData<void(const QString &, bool)>(32, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Bool, 16 },
        }}),
        // Method 'setAllRead'
        QtMocHelpers::MethodData<void(bool)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Method 'setFavorite'
        QtMocHelpers::MethodData<void(const QString &, bool)>(34, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Database, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Database::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8DatabaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8DatabaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8DatabaseE_t>.metaTypes,
    nullptr
} };

void Database::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Database *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->feedAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->feedDetailsUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->feedGroupsUpdated(); break;
        case 3: _t->feedGroupRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->entryReadChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->feedReadChanged(); break;
        case 6: _t->entryFavoriteChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->addFeed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 8: _t->addFeed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->addFeed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->importFeeds((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->exportFeeds((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->addFeedGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 13: _t->addFeedGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->removeFeedGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->setDefaultGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->editFeed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 17: _t->setRead((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 18: _t->setFeedRead((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->setAllRead((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->setFavorite((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Database::*)(const QString & )>(_a, &Database::feedAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Database::*)(const QString & , const QString & , const QString & )>(_a, &Database::feedDetailsUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Database::*)()>(_a, &Database::feedGroupsUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Database::*)(const QString & )>(_a, &Database::feedGroupRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Database::*)(const QString & , bool )>(_a, &Database::entryReadChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Database::*)()>(_a, &Database::feedReadChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Database::*)(const QString & , bool )>(_a, &Database::entryFavoriteChanged, 6))
            return;
    }
}

const QMetaObject *Database::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Database::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8DatabaseE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Database::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void Database::feedAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Database::feedDetailsUpdated(const QString & _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void Database::feedGroupsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Database::feedGroupRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Database::entryReadChanged(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void Database::feedReadChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Database::entryFavoriteChanged(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}
QT_WARNING_POP
