/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<alligatorsettings.h>)
#  include <alligatorsettings.h>
#endif
#if __has_include(<clipboard.h>)
#  include <clipboard.h>
#endif
#if __has_include(<contenthelper.h>)
#  include <contenthelper.h>
#endif
#if __has_include(<database.h>)
#  include <database.h>
#endif
#if __has_include(<entriesmodel.h>)
#  include <entriesmodel.h>
#endif
#if __has_include(<entriesproxymodel.h>)
#  include <entriesproxymodel.h>
#endif
#if __has_include(<feed.h>)
#  include <feed.h>
#endif
#if __has_include(<feedgroupsmodel.h>)
#  include <feedgroupsmodel.h>
#endif
#if __has_include(<feedsmodel.h>)
#  include <feedsmodel.h>
#endif
#if __has_include(<feedsproxymodel.h>)
#  include <feedsproxymodel.h>
#endif
#if __has_include(<fetcher.h>)
#  include <fetcher.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_alligator()
{
    qmlRegisterModule("org.kde.alligator", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<AlligatorSettings>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<Clipboard>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<ContentHelper>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<Database>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<EntriesModel>("org.kde.alligator", 254);
    qmlRegisterEnum<EntriesModel::Roles>("EntriesModel::Roles");
    qmlRegisterTypesAndRevisions<EntriesProxyModel>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<Feed>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<FeedGroupsModel>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<FeedsModel>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<FeedsProxyModel>("org.kde.alligator", 254);
    qmlRegisterTypesAndRevisions<Fetcher>("org.kde.alligator", 254);
    QMetaType::fromType<KConfigSkeleton *>().id();
    QMetaType::fromType<KCoreConfigSkeleton *>().id();
    qmlRegisterEnum<KCoreConfigSkeleton::DisambiguateConstructor>("KCoreConfigSkeleton::DisambiguateConstructor");
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    QT_WARNING_POP
    qmlRegisterModule("org.kde.alligator", 254, 254);
}

static const QQmlModuleRegistration orgkdealligatorRegistration("org.kde.alligator", qml_register_types_org_kde_alligator);
