/****************************************************************************
** Meta object code from reading C++ file 'alligatorsettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../alligatorsettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'alligatorsettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17AlligatorSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto AlligatorSettings::qt_create_metaobjectdata<qt_meta_tag_ZN17AlligatorSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AlligatorSettings",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "deleteAfterCountChanged",
        "",
        "articleFontSizeChanged",
        "articleFontUseSystemChanged",
        "deleteAfterTypeChanged",
        "sidebarCollapsedChanged",
        "deleteAfterCount",
        "isDeleteAfterCountImmutable",
        "defaultDeleteAfterCountValue",
        "articleFontSize",
        "isArticleFontSizeImmutable",
        "defaultArticleFontSizeValue",
        "articleFontUseSystem",
        "isArticleFontUseSystemImmutable",
        "defaultArticleFontUseSystemValue",
        "deleteAfterType",
        "isDeleteAfterTypeImmutable",
        "defaultDeleteAfterTypeValue",
        "sidebarCollapsed",
        "isSidebarCollapsedImmutable",
        "defaultSidebarCollapsedValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'deleteAfterCountChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'articleFontSizeChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'articleFontUseSystemChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deleteAfterTypeChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sidebarCollapsedChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'deleteAfterCount'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isDeleteAfterCountImmutable'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultDeleteAfterCountValue'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'articleFontSize'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isArticleFontSizeImmutable'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultArticleFontSizeValue'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'articleFontUseSystem'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isArticleFontUseSystemImmutable'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultArticleFontUseSystemValue'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'deleteAfterType'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isDeleteAfterTypeImmutable'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultDeleteAfterTypeValue'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'sidebarCollapsed'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isSidebarCollapsedImmutable'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultSidebarCollapsedValue'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<AlligatorSettings, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AlligatorSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<KConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AlligatorSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AlligatorSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17AlligatorSettingsE_t>.metaTypes,
    nullptr
} };

void AlligatorSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AlligatorSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->deleteAfterCountChanged(); break;
        case 1: _t->articleFontSizeChanged(); break;
        case 2: _t->articleFontUseSystemChanged(); break;
        case 3: _t->deleteAfterTypeChanged(); break;
        case 4: _t->sidebarCollapsedChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AlligatorSettings::*)()>(_a, &AlligatorSettings::deleteAfterCountChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AlligatorSettings::*)()>(_a, &AlligatorSettings::articleFontSizeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AlligatorSettings::*)()>(_a, &AlligatorSettings::articleFontUseSystemChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AlligatorSettings::*)()>(_a, &AlligatorSettings::deleteAfterTypeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AlligatorSettings::*)()>(_a, &AlligatorSettings::sidebarCollapsedChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->deleteAfterCount(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isDeleteAfterCountImmutable(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->defaultDeleteAfterCountValue(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->articleFontSize(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isArticleFontSizeImmutable(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->defaultArticleFontSizeValue(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->articleFontUseSystem(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isArticleFontUseSystemImmutable(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->defaultArticleFontUseSystemValue(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->deleteAfterType(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isDeleteAfterTypeImmutable(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->defaultDeleteAfterTypeValue(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->sidebarCollapsed(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isSidebarCollapsedImmutable(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->defaultSidebarCollapsedValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDeleteAfterCount(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setArticleFontSize(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setArticleFontUseSystem(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setDeleteAfterType(*reinterpret_cast<int*>(_v)); break;
        case 12: _t->setSidebarCollapsed(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AlligatorSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AlligatorSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AlligatorSettingsE_t>.strings))
        return static_cast<void*>(this);
    return KConfigSkeleton::qt_metacast(_clname);
}

int AlligatorSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void AlligatorSettings::deleteAfterCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AlligatorSettings::articleFontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void AlligatorSettings::articleFontUseSystemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AlligatorSettings::deleteAfterTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AlligatorSettings::sidebarCollapsedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
